
# q-expansion of newform 1584.2.a.h, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1584
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [-2], [-1], [-4], [6], [4], [6], [-6], [-8], [-10], [-6], [-8], [-6], [0], [0], [8], [4], [6], [2], [-14], [-12], [6], [14], [-6], [4], [-12], [-4], [-18], [-14], [-12], [18], [4], [6], [10], [2], [4], [12], [-6], [24], [-22], [18], [14], [-6], [4], [-8], [16], [-12], [-22], [18], [-12], [-10], [0], [30], [0], [24], [-2], [-16], [6], [-8], [6], [-20], [-18], [26], [-12], [4], [2], [36], [-4], [6], [-12], [-8], [20], [-20], [6], [0], [26], [-30], [-34], [24], [-10], [0], [26], [10], [-24], [-6], [-10], [-42], [4], [-12], [-24], [-20], [12], [4], [12], [-24], [18], [16], [20], [28], [-18], [-12], [-18], [28], [-34], [24], [30], [30], [-22], [-14], [-16], [30], [-44], [16], [-6], [4], [6], [36], [36], [-22], [14], [-30], [-24], [-20], [6], [26], [30], [28], [-4], [-8], [36], [-8], [-34], [-18], [-34], [0], [-32], [0], [42], [-8], [18], [-8], [-12], [14], [18], [8], [-54], [-20], [42], [-52], [30], [28], [-36], [-20], [-42], [-2], [-30], [-22], [18], [12], [42], [-14], [0], [-54], [30], [-56], [44], [-34], [-12], [-12], [50], [-60], [-46], [40], [6], [4], [-36], [-2], [-4], [-32], [-60], [14], [-6], [54], [12], [56], [4], [14], [0], [-10], [60], [28], [-48], [-12], [-66], [-22], [-22], [-30], [-24], [54], [-2], [26], [-46], [-48], [12], [4], [-36], [6], [-44], [-10], [12], [64], [-12], [-12], [-10], [-38], [-18], [-18], [36], [56], [58], [-6], [-8], [60], [44], [6], [-30], [-62], [-36], [14], [-8], [-74], [6], [52], [72], [-46], [6], [24], [-30], [-12], [-56], [-32], [-70], [-18], [-36], [-20], [-36], [4], [-12], [32], [-6], [42], [-10], [6], [-12], [38], [-8], [12], [14], [46], [12], [-4], [44], [18], [4], [-36], [30], [-44], [-30], [50], [28], [14], [58], [-70], [-20], [-12], [80], [-46], [60], [0], [-20], [12], [-28], [-68], [6], [14], [-54], [28], [30], [48], [24], [6], [-36], [44], [-18], [76], [72], [-60], [40], [-10], [-18], [46], [-12], [-68], [-22], [-60], [14], [-30], [-4], [-36], [-84], [18], [4], [-72], [50], [60], [72], [2], [-30], [-44], [38], [-42], [28], [6], [14], [-20], [28], [-36], [42], [-64], [72], [-74], [84], [88], [-36], [-70], [-18], [26], [34], [26], [42], [-66], [-56], [-24], [-12], [2], [-20], [-36], [-48], [40], [-94], [24], [-62], [-40], [42], [-18], [60], [-78], [42], [-16], [6], [30], [36], [4], [26], [30], [-38], [14], [24], [28], [72], [66], [-14], [-10], [-84], [-36], [-46], [66], [-10], [-96], [-54], [58], [30], [-8], [-30], [-80], [-58], [16], [-102], [-58], [36], [-72], [-68], [66], [14], [-26], [18], [100], [90], [-40], [42], [22], [42], [-66], [-2], [38], [66], [4], [84], [98], [78], [-84], [28], [50], [12], [12], [-20], [-54], [36], [-12], [32], [102], [84], [-10], [-24], [-60], [-30], [52], [-48]]
